%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Ke,Re,Ie]=element1(e_mat,e_spa,i_var,mat)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% stiffness matrix Ke and righthand side Re for quads
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% input:  e_mat = [ X1 Y1 X2 ... Y4 ]      ... material coordinates
%         e_spa = [ x1 y1 x2 ... y4 ]      ... spatial  coordinates
%         i_var = [ 4 x 1 ]                ... internal variables
%         i_var = [rho^0-rho_0^*]/rho_0^*  ... relativ change in density
%         mat   = [ ... ]                  ... material parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% output: Ke    = [ 8 x 8 ]                ... element tangent  Ke 
%         Re    = [ 8 x 1 ]                ... element residual Re
%         Ie    = [ 4 x 1 ]                ... element int var  Ie
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ie = i_var;
Re = zeros(8,1);
Ke = zeros(8,8);

nod=4;
delta = eye(2);

indx=[1;3;5;7];  ex_mat=e_mat(indx); 
indy=[2;4;6;8];  ey_mat=e_mat(indy);
 
% integration points %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
g1=0.577350269189626; w1=1;
gp(:,1)=[-g1; g1;-g1; g1];   w(:,1)=[ w1; w1; w1; w1]; 
gp(:,2)=[-g1;-g1; g1; g1];   w(:,2)=[ w1; w1; w1; w1];
wp=w(:,1).*w(:,2);      xsi=gp(:,1);      eta=gp(:,2);

% shape functions and derivatives in isoparametric space %%%%%%%%%%%%%%%
N(:,1)=(1-xsi).*(1-eta)/4;   N(:,2)=(1+xsi).*(1-eta)/4;
N(:,3)=(1+xsi).*(1+eta)/4;   N(:,4)=(1-xsi).*(1+eta)/4;
dNr(1:2:8  ,1)=-(1-eta)/4;   dNr(1:2:8  ,2)= (1-eta)/4;
dNr(1:2:8  ,3)= (1+eta)/4;   dNr(1:2:8  ,4)=-(1+eta)/4;
dNr(2:2:8+1,1)=-(1-xsi)/4;   dNr(2:2:8+1,2)=-(1+xsi)/4;
dNr(2:2:8+1,3)= (1+xsi)/4;   dNr(2:2:8+1,4)= (1-xsi)/4;

% jacobian transformation matrix %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
JT=dNr*[ex_mat;ey_mat]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% loop over all integration points %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for ip=1:4
  
  indx=[2*ip-1; 2*ip];
  detJ=det(JT(indx,:));
  if detJ<10*eps;    disp('Jacobi determinant less than zero!');   end;
  JTinv=inv(JT(indx,:));
  dNx=JTinv*dNr(indx,:);

% deformation gradient F %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  F=zeros(2,2);
  for j=1:4
    jndx=[2*j-1; 2*j];
    F=F+e_spa(jndx)'*dNx(:,j)';
  end
  
% material tangent A, stress P, int variable var %%%%%%%%%%%%%%%%%%%%%%%
  var = i_var(ip);  
  [A,P,var]=cnst_grw(F,var,mat);
  Ie(ip)=var;

% element residual %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for i=1:nod
    en=(i-1)*2;
    Re(en+ 1) =  Re(en+ 1) +(P(1,1)*dNx(1,i)' ...
                           + P(1,2)*dNx(2,i)') * detJ * wp(ip);
    Re(en+ 2) =  Re(en+ 2) +(P(2,1)*dNx(1,i)' ...
                           + P(2,2)*dNx(2,i)') * detJ * wp(ip);
  end  

% element stiffness %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for i=1:nod
  for j=1:nod
    eni=(i-1)*2;
    enj=(j-1)*2;
  
    Ke(eni+1,enj+1)=Ke(eni+1,enj+1)+(dNx(1,i)*A(1,1,1,1)*dNx(1,j) ...
                                    +dNx(1,i)*A(1,1,1,2)*dNx(2,j) ...
                                    +dNx(2,i)*A(1,2,1,1)*dNx(1,j) ...
                                    +dNx(2,i)*A(1,2,1,2)*dNx(2,j))*detJ*wp(ip);
    Ke(eni+1,enj+2)=Ke(eni+1,enj+2)+(dNx(1,i)*A(1,1,2,1)*dNx(1,j) ...
                                    +dNx(1,i)*A(1,1,2,2)*dNx(2,j) ...
                                    +dNx(2,i)*A(1,2,2,1)*dNx(1,j) ...
                                    +dNx(2,i)*A(1,2,2,2)*dNx(2,j))*detJ*wp(ip);
    Ke(eni+2,enj+1)=Ke(eni+2,enj+1)+(dNx(1,i)*A(2,1,1,1)*dNx(1,j) ...
                                    +dNx(1,i)*A(2,1,1,2)*dNx(2,j) ...
                                    +dNx(2,i)*A(2,2,1,1)*dNx(1,j) ...
                                    +dNx(2,i)*A(2,2,1,2)*dNx(2,j))*detJ*wp(ip);
    Ke(eni+2,enj+2)=Ke(eni+2,enj+2)+(dNx(1,i)*A(2,1,2,1)*dNx(1,j) ...
                                    +dNx(1,i)*A(2,1,2,2)*dNx(2,j) ...
                                    +dNx(2,i)*A(2,2,2,1)*dNx(1,j) ...
                                    +dNx(2,i)*A(2,2,2,2)*dNx(2,j))*detJ*wp(ip);
  end
  end

end
% loop over all integration points %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%